#include <gl/gl.h>
#include <stdio.h>
#include <device.h>
#include "stdmap.h"


long nmax=16384L;
double k,pi,tpi, delta;
Coord xmin= -0.5,xmax= 0.5,ymin= -0.5,ymax= 0.5;
int theMenu;
Boolean chaos;

main(argc,argv)
int argc;
char *argv[];
{
	pi = 4.0*atan(1.0);
	tpi = 2*pi;

	if(argc>=2)
		k = atof(argv[1])/tpi;
	else
		k = 0.0;
	if(argc>=3)
		delta = atof(argv[2])/tpi;
	else
		delta = 0.005/tpi;
	
#ifdef DEBUG					/*if compiled with-DDEBUG flag*/
	foreground();				/* for edge debugger*/
#endif

	prefposition(5,XMAXSCREEN-5,5,YMAXSCREEN-25);
	winopen("");
	doublebuffer();
	gconfig();

	color(BLACK);
	clear();
	color(GREEN);
	ortho2(xmin,xmax,ymin,ymax);
	
	theMenu = defpup("StdMap %t|FareyPath%l|Erase|Reset k|Fixed Point|Zoom Window|");
	qdevice(LEFTMOUSE);
	qdevice(MENUBUTTON);
	qdevice(ESCKEY);
	qdevice(UPARROWKEY);
	qdevice(DOWNARROWKEY);

	goforit();
}

goforit()
{
	int i;
	short value;
	long dev;
	double thePt[2];

	thePt[0] = .5, thePt[1] = 0.0;

while(TRUE)
{

	if(!qtest()) /*check if event que is empty*/
	{
		PlotPoints(thePt,nmax);
		swapbuffers();			/*switch buffer's and change k*/
		EraseWindow();
		k += delta;
		if(chaos)
			thePt[0] = 0.5, thePt[1] = 0.0001;
		WriteTitle(k);
	}
	else					/*deal with event*/
	{
		dev = qread(&value);
		switch(dev)
		{
			case ESCKEY:
				gexit();
				exit(0);
				break;
			case REDRAW:
				reshapeviewport();				
				EraseWindow();
				break;
			case LEFTMOUSE:
				if(value)		/*mouse down only*/
					ScreenToWorld(getvaluator(MOUSEX),getvaluator(MOUSEY),thePt);
				break;
			case MENUBUTTON:
				HandleMenu(dopup(theMenu),thePt);
				break;
			case UPARROWKEY:
				if(value) nmax += 256L;
				break;
			case DOWNARROWKEY:
				if(value) nmax -= 256L;
				break;				
			default:
				break;
		}
	}
}
}


HandleMenu(item,thePt)
int item;
double thePt[];
{


	switch(item)
	{
		case 1: /*FareyPath*/
			{
				Orbit cantorus;
				cantorus = FindCantorus();
				thePt[0] = cantorus.x;
				thePt[1] = cantorus.y;
				ringbell();
			}
			break;
		case 2:	/*Erase*/
			EraseWindow();
			break;
		case 3:	/*Reset k*/
			k = 0.0;
			break;
		case 4: /*Set for Hyperbolic Point*/
			chaos = !chaos;
			break;
		case 5:	/*Zoom Window*/
			ZoomWindow();
			break;
		default:
			break;
	}
}

#define NMAX 256

PlotPoints(thePt,n)
double thePt[];
long n;
{
	int burst,i;


	for(i=0;i<n;i+=NMAX)
	{
		bgnpoint();
		burst = min(NMAX,n-i);
		while(burst--)
		{
			thePt[1] -= k*sin(tpi*thePt[0]);
			thePt[0] += thePt[1];
			thePt[1] -= floor(thePt[1] + 0.5);
			thePt[0] -= floor(thePt[0] + 0.5);
			v2d(thePt);
		}
		endpoint();
	}
}

EraseWindow()
{
	pushattributes();
	color(BLACK);
	clear();
	popattributes();
}

ZoomWindow()
{
static long xminZoom = 5L, xmaxZoom= XMAXSCREEN-5, 
	    yminZoom = 5L, ymaxZoom = YMAXSCREEN-25;
long xorigin, yorigin, xwidth, ywidth;

	getorigin(&xorigin,&yorigin);				/*get current Window position*/
	getsize(&xwidth,&ywidth);

	winposition(xminZoom,xmaxZoom,yminZoom,ymaxZoom);	/*zoom window*/

	xminZoom = xorigin;					/*store old window position*/
	xmaxZoom = xorigin + xwidth;
	yminZoom = yorigin;
	ymaxZoom = yorigin + ywidth;
}

ScreenToWorld(sx,sy,thePt)
Screencoord sx,sy;
double thePt[];
{

	Screencoord vpl,vpr,vpb,vpt;
	long ox,oy;

	getorigin(&ox,&oy);
	sx-=ox;
	sy-=oy;

	getviewport(&vpl,&vpr,&vpb,&vpt);

	thePt[0] = (sx-vpl)*(xmax-xmin)/(vpr-vpl +1)  + xmin;
	thePt[1] = (sy-vpb)*(ymax-ymin)/(vpt-vpb +1)  + ymin;
}


WriteTitle(k)
double k;
{
	char title[255];
	sprintf(title,"standard map k= %lf",k*tpi);
	wintitle(title);
}
