/****************************Graphics*********************************
 *Map Physical Coordinates to Window Coordinates 
 *	window_x = floor((physical_x - plotSize.xmin)*plotSize.dx +0.5);
 *	window_y = floor((plotSize.ymax - physical_y)*plotSize.dy +0.5);
 *
 *	Plot scaling data is contained in the structure plotSize
 *********************************************************************/

#include "AP.h"

/********PRIVATE ROUTINES**********/
void Constrain(int curveType, Point *apt, Point bpt);
Boolean DoubleClick(long t1, long t2, Point p1, Point p2);
void DrawCurve(int curveType, Point stPt, Point endPt);
Boolean WithinWindow(double x, double y);
/*********************************/

extern WindowPtr	plot;
extern PlotData		plotSize;
extern Boolean 		hasColor;
extern Boolean		showPos;
extern int			topology;

/*************Plotting Macros************/ 
#define WindX(x)  floor(0.5 + ((x) - plotSize.xmin)*plotSize.dx)
#define WindY(y)  floor(0.5 + (plotSize.ymax - (y))*plotSize.dy)
/*NOTE These macros assume that the return value, ususally a short is 'in range"
 * Thus always clip to the window, using WithinWindow() before calling!
 ****************************************/

dPoint WindowToPhys(Point thePt)
{
	dPoint phys;
	phys.x = plotSize.xmin + ((double)thePt.h/plotSize.dx);
	phys.y = plotSize.ymax - ((double)thePt.v/plotSize.dy);
	return(phys);
}

Point PhysToWindow(dPoint z)  
{
	Point w;
    w.h = WindX(z.x);
	w.v = WindY(z.y);
	return(w);
}

Boolean WithinWindow(double x, double y)
{
	return(x>plotSize.xmin && x<plotSize.xmax && y>plotSize.ymin && y<plotSize.ymax);
}

void DoubleColors(RGBColor theColor)
{
	if(hasColor)
	{
		SetPort(plotSize.backbuf);
		RGBForeColor(&theColor);
		SetPort(plot);
		RGBForeColor(&theColor);
	}
}

void SingleColors(RGBColor theColor)
{
	if(hasColor) RGBForeColor(&theColor);
}

void DoublePoint(double x, double y)
{
	if(WithinWindow(x,y))
	{
		int h,v;
		h = WindX(x);
		v = WindY(y);
	
		SetPort(plotSize.backbuf);
		MoveTo(h,v);						
		Line(0,0);
	
		SetPort(plot); 
		MoveTo(h,v);						
		Line(0,0);
	}
}

void SinglePoint(double x, double y)
{
	if(WithinWindow(x,y))
	{
		int h,v;
		h = WindX(x);
		v = WindY(y);
		MoveTo(h,v);						
		Line(0,0);
	}
}

void PlotBox(double x, double y)
{
	if(WithinWindow(x,y))
	{
		Point v;
		dPoint z;
		Rect r;
		z.x = x; z.y = y;
		v = PhysToWindow(z);
		SetRect(&r,v.h-2,v.v-2,v.h+2,v.v+2);
		if(plotSize.backbuf != nil)
		{
			SetPort(plotSize.backbuf);
			PaintRect(&r);
		}	
		SetPort(plot);
		PaintRect(&r);
	}
}

void PlotLine(dPoint st, dPoint fn)
{
	Point v;
	
	v = PhysToWindow(st);
	MoveTo(v.h,v.v);
	v = PhysToWindow(fn);
	LineTo(v.h,v.v);
}

void PlotAxes(void)
{
	double xo,yo;
	dPoint st, fn;
	Point apt;
	RGBColor newColor, oldColor;
	
	SetPort(plot);
	   
	if(plotSize.xmin < 0 && plotSize.xmax > 0)						//put axis on coordinate axis if possible
		xo = 0.0;
	else
		xo = plotSize.xmin + 0.5*(plotSize.xmax-plotSize.xmin);		//otherwise center it
	if(plotSize.ymin < 0 && plotSize.ymax > 0)
		yo = 0.0;
	else
		yo = plotSize.ymin + 0.5*(plotSize.ymax-plotSize.ymin);
		
	if(hasColor)
	{
		GetForeColor(&oldColor);
		GetBackColor(&newColor);
		InvertColor(&newColor);
		RGBForeColor(&newColor);
	}

	st.x = xo, st.y = plotSize.ymin;
	fn.x = xo, fn.y = plotSize.ymax;
	PlotLine(st,fn);

	apt = PhysToWindow(st);
	MoveTo(apt.h+1, apt.v-3);
	DrawString(plotSize.ymnl);
	apt = PhysToWindow(fn);
	MoveTo(apt.h+1, apt.v+plotSize.labelHeight);
	DrawString(plotSize.ymxl);

	st.x = plotSize.xmin, st.y = yo;
	fn.x = plotSize.xmax, fn.y = yo;
	PlotLine(st,fn);

	apt = PhysToWindow(st);
	MoveTo(apt.h, apt.v+plotSize.labelHeight);
	DrawString(plotSize.xmnl);
	apt = PhysToWindow(fn);
	MoveTo(apt.h-plotSize.labelWidth, apt.v+plotSize.labelHeight);
	DrawString(plotSize.xmxl);
	
	if(hasColor)
		RGBForeColor(&oldColor);
}


void DragPath(Point *pathPts, int *max)  		// given an pathPts[max], with element 0 set to first pt
{										 		// add a point for each mouse click
	Point oldPt,newPt;					 		// until double click
	long time;
	int i;
	EventRecord theEvent;
	Boolean constrain;
	
				
	SetPort(plot);		
    PenMode(patXor);

	time=TickCount();
	constrain = isPressed(SHIFT);					// shift down so constrain drawing

	for(i=1, oldPt=pathPts[0]; i< *max ; i++, time = theEvent.when, oldPt = newPt)
	{	
	    do
    	{
    		GetNextEvent(mDownMask,&theEvent);
    		GetMouse(&newPt);
    		if(constrain)
    			Constrain(LINE,&newPt,pathPts[i-1]);
			if(oldPt.h!=newPt.h || oldPt.v != newPt.v)		//only if mouse moved
			{
				DrawCurve(LINE,pathPts[i-1],oldPt);			//erase old curve
				DrawCurve(LINE,pathPts[i-1],newPt);			//draw new curve
				oldPt=newPt;
				constrain = isPressed(SHIFT);				// shift down so constrain drawing
			}
			if(showPos)										//update position indicator
			{
			    PenMode(patCopy);
				UpdatePosition();
			    PenMode(patXor);
			}
		} while(theEvent.what!=mouseDown);
		
		if(DoubleClick(theEvent.when,time,newPt,pathPts[i-1]))
			break;
		pathPts[i] = newPt;
	}
	*max = i;
	
	for(i=1;i<*max;i++)
	{
		DrawCurve(LINE,pathPts[i-1],pathPts[i]);			//erase Curve
	}
	PenMode(patCopy);
}


Point DragPoint(Point anchor, int type)						// drag a curve of 'type' from anchor
{															// until mouse released
	Point endPt, oldPt;										// returning the release point
	Boolean constrain;

	constrain = isPressed(SHIFT);							// shift down so constrain drawing	
	SetPort(plot);
	
    PenMode(patXor);
    oldPt = anchor;
    do
	{
		GetMouse(&endPt);
		if(constrain)
			Constrain(type, &endPt,anchor);
			
		if(oldPt.h!=endPt.h || oldPt.v != endPt.v)			//only if mouse moved
		{
			DrawCurve(type,anchor,oldPt);					//erase old curve
			DrawCurve(type,anchor,endPt);					//draw new curve
			oldPt=endPt;			
		}
		if(showPos)											//update position indicator
		{
		    PenMode(patCopy);
			UpdatePosition();
		    PenMode(patXor);
		}
	}while(StillDown());
	
	DrawCurve(type,anchor,oldPt);							//erase old curve
	PenMode(patCopy);
	return(endPt);
}

void DrawCurve(int curveType, Point stPt, Point endPt)
{
	Rect DragRect;
	
	SetRect(&DragRect, min(stPt.h,endPt.h),min(stPt.v,endPt.v),max(stPt.h,endPt.h),max(stPt.v,endPt.v));
	switch(curveType)
	{		
		default:
		case BOX:		
			FrameRect(&DragRect);
		break;
		case CIRCLE:
			FrameOval(&DragRect);
		break;
		case LINE:
			MoveTo(stPt.h, stPt.v);
			LineTo(endPt.h,endPt.v);
		break;
	}
}

Boolean DoubleClick(long t1, long t2, Point p1, Point p2)
{	
	return(((t1-t2) < GetDblTime()) && (abs(p1.h-p2.h)+abs(p1.v-p2.v))<4);
}

void Constrain(int curveType, Point *apt, Point bpt)  //make apt-bpt vertical, diagonal or horizontal only
{
	int dx,dy;
	dx = apt->h-bpt.h;
	dy = bpt.v-apt->v;
	if(curveType!=LINE)
		apt->v = bpt.v - abs(dx)*sign(dy);
	else if(abs(dx) < 0.5*abs(dy))
		apt->h = bpt.h;
	else if(abs(dx) > 2.0*abs(dy))
		apt->v = bpt.v;
	else 
		apt->v = bpt.v - abs(dx)*sign(dy);
}

void PlotSymmetry()
{
	double dy, y;
	ifnumber x;
	RGBColor aColor;
	int type;
	
	dy = 1.0/plotSize.dy;
	aColor = GetTheColor();
	for(type = FIXS; type<=FIXTSR;type++)
	{
		ColorList(&aColor);
		SetColors(aColor);
		for(y=plotSize.ymin; y<plotSize.ymax; y+=dy)
		{
				x = Symmetry(y,type,0);
				if(topology==CYLINDER)
					x.f -= floor(x.f+0.5);
				PlotPoint(x.f,y);
		}
	}
}