#include <gl/gl.h>
#include <stdio.h>
#include <device.h>
#include "stdmap.h"

#define NMAX 256L
#define DELTAK 0.02

double k,pi,tpi;
long xminZoom, xmaxZoom,yminZoom, ymaxZoom;
PlotData plotSize;
int theMenu;
short theColor,  maxColor,  minColor;

main(argc,argv)
int argc;
char *argv[];
{
    int i;
    short value;
    long dev;
    double thePt[2]={0.4999, 0.0};
    Point msPt;

    if(argc==2)
	    k = atof(argv[1]);
    else
	    k = 0.97163540631;

    initialize();

    while(TRUE)
    {
	if(!qtest()) /*check if event que is empty*/
		PlotStd(thePt,NMAX);
	else					/*deal with event*/
	{
	    dev = qread(&value);
	    switch(dev)
	    {
		case ESCKEY:
		    gexit();
		    exit(0);
		    break;
		case REDRAW:
		    reshapeviewport();				
		    EraseWindow();
		    break;
		case LEFTMOUSE:
		    if(value)		/*mouse down only*/
		    {
			GetMouse(&msPt);
			GlobalToLocal(msPt, thePt);
			color(theColor++);
			if(theColor > maxColor)
			    theColor = minColor;
		    }
		    break;
		case MIDDLEMOUSE:
		    if(value)		/*mouse down only*/
		    {	DragScale(1);
			color(theColor);
		    }
		    break;
		case MENUBUTTON:
		    HandleMenu(dopup(theMenu));
		    break;
		case UPARROWKEY:
		    if(value)
		    {
			    k+= DELTAK/tpi;
			    WriteTitle();
			    EraseWindow();
		    }
		    break;
		case DOWNARROWKEY:
		    if(value)
		    {
			    k -= DELTAK/tpi;
			    WriteTitle();
			    EraseWindow();
		    }
		    break;
		default:
		    break;
	    }
	}
    }
}

void HandleMenu(int item)
{
    switch(item)
    {
	case 1: /*FareyPath*/
	{
	    Orbit cantorus;
	    double thePt[2];
	    cantorus = FindCantorus();
	    thePt[0] = cantorus.x;
	    thePt[1] = cantorus.y;
	    PlotRects(thePt,cantorus.q);
	    thePt[0] = -cantorus.x;
	    thePt[1] = -cantorus.y;
	    PlotRects(thePt,cantorus.q);
	    ringbell();
	}
	break;
	case 2:	/*Erase*/
	    EraseWindow();
	    break;
	case 3: /*reset Scale*/
	    ReScale(-0.5, 0.5, -0.5, 0.5);
	case 4:	/*reset Color*/
	    theColor=minColor;
	    break;
	case 5:	/*Zoom Window*/
	    ZoomWindow();
	    break;
	default:
	    break;
    }
}

void PlotStd(double *thePt,long n)
{
    int burst,i;
    for(i=0;i<n;i+=NMAX)
    {
	burst = min(NMAX,n-i);
	bgnpoint();
	while(burst--)
	{
	    thePt[1] -= k*sin(tpi*thePt[0]);
	    thePt[0] += thePt[1];
	    thePt[1] -= floor(thePt[1] + 0.5);
	    thePt[0] -= floor(thePt[0] + 0.5);
	    v2d(thePt);
	}
	endpoint();
    }
}

void PlotHenon(double *thePt,long n)
{
	int burst,i;
	double y;

	for(i=0;i<n;i+=NMAX)
	{
		bgnpoint();
		burst = min(NMAX,n-i);
		while(burst--)
		{
			y = thePt[1];
			thePt[1] = thePt[0];
			thePt[0] = thePt[0]*(k-thePt[0])- y;
			thePt[0] -= 2*floor(0.5*thePt[0] +0.5);
			thePt[1] -= 2*floor(0.5*thePt[1] + 0.5);
			v2d(thePt);
		}
		endpoint();
	}
}

#define DY 0.005
void PlotRects(double *thePt,long n)
{
	
	pushattributes();
	color(CYAN);
	while(n--)
	{
		rectf(thePt[0],thePt[1],thePt[0]+DY,thePt[1]+DY);
		thePt[1] -= k*sin(tpi*thePt[0]);
		thePt[0] += thePt[1];
		thePt[1] -= floor(thePt[1] + 0.5);
		thePt[0] -= floor(thePt[0] + 0.5);
	}
	popattributes();
}	


void EraseWindow()
{
	pushattributes();
	color(BLACK);
	clear();
	popattributes();
}

void ZoomWindow()
{
    long xorigin, yorigin, xwidth, ywidth;

    getorigin(&xorigin,&yorigin);			/*get current Window position*/
    getsize(&xwidth,&ywidth);

    winposition(xminZoom,xmaxZoom,yminZoom,ymaxZoom);	/*zoom window*/

    xminZoom = xorigin;					/*store old window position*/
    xmaxZoom = xorigin + xwidth;
    yminZoom = yorigin;
    ymaxZoom = yorigin + ywidth;
}

void WriteTitle()
{
	char title[255];
	sprintf(title,"Std Map k= %lf (%.3lg, %.3lg)x((%.3lg, %.3lg)",k*tpi,plotSize.xmin,plotSize.ymin,plotSize.xmax,plotSize.ymax);
	wintitle(title);
}

void initialize()
{
    char title[30];
    int nplanes;
    
    pi = 4.0*atan(1.0);
    tpi = 2*pi;
    k /= tpi;

#ifdef DEBUG
    foreground();	    //for edge debugger and printf
#endif
    if(system("colorEdit 128 3")==256)
	printf("colorEdit is available for anonymous ftp from amath.colorado.edu\n");
    winopen("");

    nplanes = getgdesc(GD_BITS_NORM_SNG_CMODE);
    maxColor = 2<<(nplanes-1);
    if(getgdesc(GD_BITS_OVER_SNG_CMODE))	//set up the overlay planes
    {
	overlay(2);  
	gconfig();
	drawmode(OVERDRAW);
	mapcolor(0,0,0,0);
	mapcolor(1,255, 255, 255);
	drawmode(NORMALDRAW);
    } else
	printf("Warning--no overlay planes--enlarge won't work\n");
    if(nplanes > 8)
	theColor = minColor = 2<<(nplanes-3);
    else
	theColor = minColor = 2<<(nplanes-2);


    ReScale(-0.5,0.5,-0.5,0.5);
    color(theColor++);
 
    xminZoom=yminZoom = 10L;
    xmaxZoom = getgdesc(GD_XPMAX)-10L;
    ymaxZoom = getgdesc(GD_YPMAX)-35L;

    theMenu = defpup("StdMap %t|FareyPath%l|Erase|reset Scale|reset Colors|Zoom Window|");
    qdevice(LEFTMOUSE);
    qdevice(MIDDLEMOUSE);
    qdevice(MENUBUTTON);
    qdevice(ESCKEY);
    qdevice(UPARROWKEY);
    qdevice(DOWNARROWKEY);
	
}
